#ifndef __IN_EVENT_H__
#define __IN_EVENT_H__
//****************************************************************************
//**
//**    IN_EVENT.H
//**    Header - Input - Common Event Interface
//**
//**    Copyright (c) 1998 Chris Hargrove
//**    Notice: Usage of any code in this file is subject to the rules
//**            described in the LICENSE.TXT file included in this directory.
//**            Reading, compiling, or otherwise using this code constitutes
//**            automatic acceptance of the rules in said text file.
//**
//****************************************************************************
//============================================================================
//    INTERFACE REQUIRED HEADERS
//============================================================================
#include "sys_defs.h"

//============================================================================
//    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================
// input event type
typedef enum
{
	INEV_NONE, // invalid
	INEV_MOUSEMOVE, // mouse position has changed
	INEV_PRESS, // key has changed from released to pressed
	INEV_DRAG, // key is still being pressed
	INEV_RELEASE, // key has changed from pressed to released
	
	INEV_NUMTYPES
} in_eventType_t;

// input event flags
enum
{
	// first three flags hold status of modifier keys
	INEVF_SHIFT		= BITF(0),
	INEVF_ALT		= BITF(1),
	INEVF_CTRL		= BITF(2)
};

// key constants
enum
{
	// --------------------
	// characters 0-127 map to their ascii characters
	// --------------------

	INKEY_NULL			= 0x00000000,
    INKEY_BACKSPACE		= 0x00000008,
    INKEY_TAB			= 0x00000009,
    INKEY_ENTER			= 0x0000000D,
    INKEY_ESCAPE		= 0x0000001B,
    INKEY_SPACE			= 0x00000020, // same as ' ', here for convenience
	// regular characters don't need constants here; use 'a' 'b' etc.

	// --------------------
	// characters 128-255 are used for extended keys
	// --------------------
    
	// modifier keys
    INKEY_LEFTSHIFT		= 0x00000080,
    INKEY_RIGHTSHIFT,
	INKEY_LEFTCTRL,
	INKEY_RIGHTCTRL,
    INKEY_LEFTALT,
    INKEY_RIGHTALT,
	// arrow keys
	INKEY_LEFTARROW,
    INKEY_RIGHTARROW,
    INKEY_UPARROW,
    INKEY_DOWNARROW,
    // function keys
	INKEY_F1,
    INKEY_F2,
    INKEY_F3,
    INKEY_F4,
    INKEY_F5,
    INKEY_F6,
    INKEY_F7,
    INKEY_F8,
    INKEY_F9,
    INKEY_F10,
    INKEY_F11,
    INKEY_F12,
	// cursor control keys
    INKEY_INS,
    INKEY_DEL,
    INKEY_HOME,
    INKEY_END,
    INKEY_PGUP,
    INKEY_PGDN,
	// numeric keypad
    INKEY_NUMSLASH,
    INKEY_NUMSTAR,
    INKEY_NUMMINUS,
    INKEY_NUMPLUS,
    INKEY_NUMENTER,
    INKEY_NUMPERIOD,
    INKEY_NUM0,
    INKEY_NUM1,
    INKEY_NUM2,
    INKEY_NUM3,
    INKEY_NUM4,
    INKEY_NUM5,
    INKEY_NUM6,
    INKEY_NUM7,
    INKEY_NUM8,
    INKEY_NUM9,
    // locks and misc keys
	INKEY_NUMLOCK,
    INKEY_CAPSLOCK,
    INKEY_SCROLLLOCK,
    INKEY_PRINTSCRN,
    INKEY_PAUSE,
	// windows keys deliberately not listed (i don't believe in
	// keyboards specially rigged for a single operating system)

	// --------------------
	// characters 256 and up used for mouse buttons etc.
	// --------------------

	INKEY_MOUSELEFT		= 0x00000100,
	INKEY_MOUSERIGHT,
	INKEY_MOUSEMIDDLE,

	// --------------------
	INKEY_NUMKEYS
};

//============================================================================
//    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
//============================================================================
//============================================================================
//    INTERFACE STRUCTURES / UTILITY CLASSES
//============================================================================

// input event structure
typedef struct
{
	in_eventType_t eventType; // INEV type of event
	dword key; // INKEY_ value if event is a key-related event
	dword flags; // INEVF_ flag combination
	sdword mouseX, mouseY; // mouse position within set limits
	sdword mouseFreeX, mouseFreeY; // mouse position without limits
	sdword mouseDeltaX, mouseDeltaY; // mouse change since last context event
	float pressTimeDelta; // time since key was first pressed, for key events
	dword eventIndex; // index of event, incremented for each event sent
	float frameTimeStamp; // timestamp of event at frame resolution
    float lastPressEventDelta; // time since key last had a press event
} in_event_t;

//============================================================================
//    INTERFACE DATA DECLARATIONS
//============================================================================
//============================================================================
//    INTERFACE FUNCTION PROTOTYPES
//============================================================================
//============================================================================
//    INTERFACE OBJECT CLASS DEFINITIONS
//============================================================================
//============================================================================
//    INTERFACE TRAILING HEADERS
//============================================================================

//****************************************************************************
//**
//**    END HEADER IN_EVENT.H
//**
//****************************************************************************
#endif // __IN_EVENT_H__
