#ifndef __RES_MAN_H__
#define __RES_MAN_H__
//****************************************************************************
//**
//**    RES_MAN.H
//**    Header - Resource Manager
//**
//**    Copyright (c) 1998 Chris Hargrove
//**    Notice: Usage of any code in this file is subject to the rules
//**            described in the LICENSE.TXT file included in this directory.
//**            Reading, compiling, or otherwise using this code constitutes
//**            automatic acceptance of the rules in said text file.
//**
//****************************************************************************
//============================================================================
//    INTERFACE REQUIRED HEADERS
//============================================================================
#include "sys_defs.h"
//============================================================================
//    INTERFACE DEFINITIONS / ENUMERATIONS / SIMPLE TYPEDEFS
//============================================================================
// resource id handle
typedef unsigned long res_id_t;

// resource cache types
typedef enum
{
    RES_CACHE_TEMP, // temporary resource, cache out each frame
    RES_CACHE_LEVEL, // level resource, cache out when level change occurs
    RES_CACHE_GAME, // game static resource, never cache out

    RES_CACHE_NUMTYPES
} res_cacheType_t;

// handler function typedefs
typedef void*(*res_loaderFunc_t)(char* fileName);
typedef void(*res_freeFunc_t)(res_id_t id);

//============================================================================
//    INTERFACE CLASS PROTOTYPES / EXTERNAL CLASS REFERENCES
//============================================================================
//============================================================================
//    INTERFACE STRUCTURES / UTILITY CLASSES
//============================================================================
//============================================================================
//    INTERFACE DATA DECLARATIONS
//============================================================================
//============================================================================
//    INTERFACE FUNCTION PROTOTYPES
//============================================================================

// init/shutdown/frame
void RES_Init();
void RES_Shutdown();
void RES_Frame();

// resource registration
res_id_t RES_Register(char* fileName, res_cacheType_t cacheType,
                      res_loaderFunc_t loadFunc,
                      res_freeFunc_t freeFunc); // register a resource file

// resource retrieval
void* RES_PtrForId(res_id_t id, dword* size); // retrieve resource data
char* RES_FileNameForId(res_id_t id); // retrieve resource filename

// cache control
void RES_LevelChange(); // level change, cache out level resources

// loader function memory allocation, only works in loaders
void* RES_Malloc(dword size);

//============================================================================
//    INTERFACE OBJECT CLASS DEFINITIONS
//============================================================================

// convenience resource proxy template
template <class T> class res_proxy_t
{
private:
    res_id_t id;
public:
    res_proxy_t(res_id_t inId)
    {
        id = inId;
    }
    res_proxy_t(char* fileName, res_cacheType_t cacheType,
        res_loaderFunc_t loadFunc, res_freeFunc_t freeFunc)
    {
        id = RES_Register(fileName, cacheType, loadFunc, freeFunc);
    }
    ~res_proxy_t()
    {
    }
    T& operator*()
    {
        return(*((T*)RES_PtrForId(id)));
    }
    T* operator->()
    {
        return((T*)RES_PtrForId(id));
    } 
};

//============================================================================
//    INTERFACE TRAILING HEADERS
//============================================================================

//****************************************************************************
//**
//**    END HEADER RES_MAN.H
//**
//****************************************************************************
#endif // __RES_MAN_H__
